/**************************************************************
 * ObjFactory.cpp 
 *
 * Copyright (C) 2002 - Pacific Star Technologies Ltd
 * All Rights Reserved. Unauthorized dissemination of this file
 * is strictly prohibited.
 *
 * Created: Nov 04, 2002
 *************************************************************/
#include "ObjFactory.h"
#include "Helper.h"
#include "Module.h"
#include "PropPage.h"

#pragma warning(disable : 4355)

CModFactory::CModFactory(IUnknown *pOuter)
: child(pOuter, this)
{
  refCnt = 0;
  out = pOuter;
}

CModFactory::~CModFactory()
{
}

CModFactory::CModFactImpl::CModFactImpl(IUnknown *pOuter, 
    CModFactory *pBack) 
{
  parent = pBack;
  if (NULL == pOuter) {
    outer = pBack;
  } else {
    outer = pOuter;
  }
  refCnt = 0;
}
  
CModFactory::CModFactImpl::~CModFactImpl()
{
}


STDMETHODIMP 
CModFactory::QueryInterface(REFIID iid, void ** ppv)
{
  HRESULT hr = E_NOINTERFACE;
  *ppv = NULL;
  if (IID_IUnknown == iid) {
    *ppv = static_cast<IUnknown *>(this);
  } else if (IID_IClassFactory == iid) {
    *ppv = static_cast<IClassFactory *>(&child);
  }

  if (*ppv != NULL) {
    static_cast<IUnknown *>(*ppv)->AddRef();
    hr = NOERROR;
  }
  return hr;
}

STDMETHODIMP
CModFactory::CModFactImpl::QueryInterface(REFIID iid, void ** ppv)
{
  return outer->QueryInterface(iid, ppv);
}

STDMETHODIMP_(ULONG)
CModFactory::AddRef()
{
  return ++refCnt;
}

STDMETHODIMP_(ULONG)
CModFactory::CModFactImpl::AddRef()
{
  refCnt++;
  return outer->AddRef();
}

STDMETHODIMP_(ULONG)
CModFactory::Release()
{
  refCnt--;

  if (0 == refCnt) {
    refCnt++;
    delete this;
    CSvrState::state->DecObjs();
  }
  return refCnt;
}

STDMETHODIMP_(ULONG)
CModFactory::CModFactImpl::Release()
{
  refCnt--;
  outer->Release();
  return refCnt;
}


STDMETHODIMP 
CModFactory::CModFactImpl::CreateInstance
(IUnknown *pOuter, REFIID iid, void ** ppv)
{
  HRESULT hr = E_FAIL;
  *ppv = NULL;

  //TODO: Adjust to name of your module class
  ModuleName *pObj = new ModuleName(pOuter);
  
  if (NULL != pObj) {
    hr = pObj->QueryInterface(iid, ppv);
    if (FAILED(hr)) {
      delete pObj;
    } else {
      CSvrState::state->IncObjs();
    }
  } else {
    hr = E_OUTOFMEMORY;
  }
  return hr;
}
  
STDMETHODIMP 
CModFactory::CModFactImpl::LockServer(BOOL lock)
{
  if (lock) {
    CSvrState::state->IncLocks();
  } else {
    CSvrState::state->DecLocks();
  }
  return S_OK; 
}


CPropFactory::CPropFactory(IUnknown *pOuter, CLSID which)
  : child(pOuter, this)
{
  outer = pOuter;
  refCnt = 0;
  page = which;
}

CPropFactory::~CPropFactory()
{
  CSvrState::state->DecObjs();
}

STDMETHODIMP
CPropFactory::QueryInterface(REFIID iid, void **ppv)
{
  HRESULT hr = E_NOINTERFACE;
  *ppv = NULL;

  if (IID_IUnknown == iid) {
    *ppv = this;
  } else if (IID_IClassFactory == iid) {
    *ppv = &child;
  }
  
  if (*ppv) {
    ((IUnknown *)*ppv)->AddRef();
    hr = S_OK;
  }
  return hr;
}

STDMETHODIMP_(ULONG)
CPropFactory::AddRef()
{
  return ++refCnt;
}

STDMETHODIMP_(ULONG)
CPropFactory::Release()
{
  UINT cnt = --refCnt;

  if (0 == cnt) {
    refCnt++;
    delete this;
  }

  return cnt;
}

CPropFactory::CPropImpl::CPropImpl(IUnknown *pOuter, CPropFactory *pBack)
{
  parent = pBack;
  if (NULL != pOuter)
    outer = pOuter;
  else
    outer = pBack;
}

CPropFactory::CPropImpl::~CPropImpl()
{
}

STDMETHODIMP
CPropFactory::CPropImpl::QueryInterface(REFIID iid, void **ppv)
{
  return outer->QueryInterface(iid, ppv);
}

STDMETHODIMP_(ULONG)
CPropFactory::CPropImpl::AddRef()
{
  return outer->AddRef();
}

STDMETHODIMP_(ULONG)
CPropFactory::CPropImpl::Release()
{
  return outer->Release();
}

STDMETHODIMP 
CPropFactory::CPropImpl::CreateInstance
(IUnknown *pOuter, REFIID iid, void ** ppv)
{
  HRESULT hr = E_FAIL;
  *ppv = NULL;

  if (NULL != pOuter && iid != IID_IUnknown) {
    hr = CLASS_E_NOAGGREGATION;
  } else {
    PropPage *pObj = NULL;

    // TODO: Add construction of property pages
    if (CLSID_PropPage1 == parent->page)
      pObj = new PropPage1();

    if (NULL != pObj) {
      hr = pObj->QueryInterface(iid, ppv);
      if (FAILED(hr)) {
        delete pObj;
      } else {
        CSvrState::state->IncObjs();
      }
    } else {
      hr = E_OUTOFMEMORY;
    }
  }
  return hr;
}

STDMETHODIMP 
CPropFactory::CPropImpl::LockServer(BOOL lock)
{
  if (lock) {
    CSvrState::state->IncLocks();
  } else {
    CSvrState::state->DecLocks();
  }
  return S_OK; 
}


