/**************************************************************
 * PropPage.cpp 
 *
 * Copyright (C) 2002 - Pacific Star Technologies Ltd 
 * All Rights Reserved. Unauthorized dissemination of this file
 * is strictly prohibited.
 *
 * Created: Nov 04, 2002 
 *************************************************************/

#include "PropPage.h"
#include "Helper.h"
#include "StrCast.h"
#include "resource.h"
#include "RobotGuid.h"
#include "RobotLib.h"
#include <tchar.h>
#include <stdio.h>
#include <commctrl.h>
#include <shlobj.h>

#pragma warning (disable : 4018)

static
INT_PTR CALLBACK
DoDialog(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  switch (msg) {
    case WM_INITDIALOG:
      SetWindowLongPtr(hwnd, DWLP_USER, lParam);
      break;
    case WM_STYLECHANGING:
      if (wParam == GWL_EXSTYLE) {
        LPSTYLESTRUCT lpss = (LPSTYLESTRUCT)lParam;
        lpss->styleNew |= WS_EX_CONTROLPARENT;
        return 0;
      }
  }
  PropPage *page = (PropPage *)GetWindowLongPtr(hwnd, DWLP_USER);
  if (NULL != page) 
    return page->DlgProc(hwnd, msg, wParam, lParam);
  return DefWindowProc(hwnd, msg, wParam, lParam);
}

PropPage::PropPage(UINT templ, UINT cap, UINT doc, UINT ctx)
{
  dirty = false;
  refCnt = 0;
  site = NULL;
  page = NULL;
  module = NULL;
  pRobot = NULL;

  rsrc = MAKEINTRESOURCE(templ);
  
  HINSTANCE inst = CSvrState::state->dllInst;
  TCHAR temp[64];
  
  LoadString(inst, cap, temp, 64);
  T2W(temp, title, 32);

  LoadString(inst, doc, temp, 64);
  T2W(temp, docString, 64);

  helpCtx = ctx;
}

PropPage::~PropPage()
{
  SAFE_REL(module);
  SAFE_REL(pRobot);
  CSvrState::state->DecObjs();
}

STDMETHODIMP 
PropPage::QueryInterface(REFIID iid, void **ppv)
{
  HRESULT hr = E_NOINTERFACE;
  *ppv = NULL;
  
  if (IID_IUnknown == iid || IID_IPropertyPage == iid) {
    *ppv = this;
    AddRef();
    hr = S_OK;
  }
  return hr;
}

STDMETHODIMP_(ULONG) 
PropPage::AddRef()
{
  return ++refCnt;
}
 
STDMETHODIMP_(ULONG) 
PropPage::Release()
{
  ULONG cnt = --refCnt;
  if (cnt == 0) {
    refCnt++;
    delete this;
  }
  return cnt;
}

STDMETHODIMP 
PropPage::SetPageSite(IPropertyPageSite *pageSite)
{
  if (pageSite) {
    if (site) return E_UNEXPECTED;
    site = pageSite;
    site->AddRef();
  } else {
    if (NULL == site) return E_UNEXPECTED;
    site->Release();
    site = NULL;
  }
  return S_OK;
}

STDMETHODIMP 
PropPage::GetPageInfo(PROPPAGEINFO *pageInfo)
{
  pageInfo->cb = sizeof(PROPPAGEINFO);
  pageInfo->pszTitle = (LPWSTR)CoTaskMemAlloc((wcslen(title)+1) * sizeof(WCHAR));
  wcscpy(pageInfo->pszTitle, title);

  pageInfo->size.cx = 360; 
  pageInfo->size.cy = 250; 
  pageInfo->pszDocString = (LPWSTR)CoTaskMemAlloc((wcslen(docString)+1) * sizeof(WCHAR));
  wcscpy(pageInfo->pszDocString, docString);

  // TODO: You may wish to set the path to the help file here
  pageInfo->pszHelpFile = NULL;
  pageInfo->dwHelpContext = 0;
  return S_OK;
}

STDMETHODIMP 
PropPage::Activate(HWND hwnd, LPCRECT rect, BOOL modal)
{
  if (page) return E_UNEXPECTED;
  page = CreateDialogParam(CSvrState::state->dllInst, rsrc, hwnd, DoDialog, (LPARAM)this);
  MoveWindow(page, rect->left, rect->top, rect->right-rect->left, rect->bottom-rect->top, TRUE);
  ShowWindow(page, SW_SHOWNORMAL);
  return S_OK;
}

STDMETHODIMP 
PropPage::Deactivate()
{
  if (NULL == page) return E_UNEXPECTED;
  DWORD style = GetWindowLong(page, GWL_EXSTYLE);
  style &= ~(WS_EX_CONTROLPARENT);
  HWND temp = page;
  page = NULL;
  SetWindowLong(temp, GWL_EXSTYLE, style);
  DestroyWindow(temp);
  return S_OK;
}

STDMETHODIMP 
PropPage::SetObjects(ULONG numObjs, IUnknown **pObjs)
{
  ULONG i;

  if (NULL == pObjs) return E_POINTER;
  if (0 == numObjs) {
    // release
    SAFE_REL(module);
    SAFE_REL(pRobot);
    return S_OK;
  } else {
    SAFE_REL(module);
    SAFE_REL(pRobot);

    for (i = 0; i < numObjs; i++) {
      if (!module && pObjs[i]->QueryInterface(IID_IRbtMotionDetect, (void **)&module) == S_OK) 
        continue;
      if (!pRobot && pObjs[i]->QueryInterface(IID_IRobot, (void **)&pRobot) == S_OK) 
        continue;
    }
    
    if (module && pRobot)
      return S_OK;
    else
      return E_NOINTERFACE;
  }
}

STDMETHODIMP 
PropPage::Show(UINT nCmd)
{
  ShowWindow(page, nCmd);
  return S_OK;
}

STDMETHODIMP 
PropPage::Move(LPCRECT rect)
{
  if (NULL == rect) return E_POINTER;
  if (MoveWindow(page, rect->left, rect->top, rect->right-rect->left, rect->bottom-rect->top, TRUE))
    return S_OK;
  else
    return E_UNEXPECTED;
}

STDMETHODIMP 
PropPage::IsPageDirty()
{
  return (dirty ? S_OK : S_FALSE);
}
 
STDMETHODIMP 
PropPage::Help(LPCOLESTR helpDir)
{
  return E_NOTIMPL;
}

STDMETHODIMP 
PropPage::TranslateAccelerator(LPMSG pMsg)
{
  return E_NOTIMPL;
}

// TODO: IDS_PAGE1TITLE and IDS_PAGE1DOC should be strings in the string table that
// TODO: contain the title and description of the property page
PropPage1::PropPage1()
: PropPage(IDD_PROPPAGE1, IDS_PAGE1TITLE, IDS_PAGE1DOC, 0)
{
}

PropPage1::~PropPage1()
{
}

STDMETHODIMP 
PropPage1::Apply()
{
  if (NULL == module) return E_UNEXPECTED;

  dirty = false;
  return S_OK;
}

INT_PTR 
PropPage1::DlgProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  switch (msg) {
    case WM_INITDIALOG:
      {
        page = hwnd;
        break;
      }
    case WM_COMMAND:
      break;
  }
  return DefWindowProc(hwnd, msg, wParam, lParam);
}
